from django.contrib import admin

from .forms import *
from .models import *

try:
    import genericm2m
except ImportError:
    genericm2m = None

try:
    import taggit
except ImportError:
    taggit = None



models = [FkModel, OtoModel, MtmModel, GfkModel]

if genericm2m:
    models.append(GmtmModel)

if taggit:
    models.append(TaggitModel)


for model in models:
    ModelForm = autocomplete_light.modelform_factory(model,
        exclude=['for_inline', 'noise'])

    Inline = type(str('%sInline') % model.__name__, (admin.TabularInline,), {
        'form': ModelForm, 'model': model, 'fk_name': 'for_inline'})

    ModelAdmin = type(str('%sAdmin' % model.__name__), (admin.ModelAdmin,), {
        'form': ModelForm, 'inlines': [Inline]})

    admin.site.register(model, ModelAdmin)



from __future__ import unicode_literals

import six
from django import forms
from django.db import models
from django.db.models.query import QuerySet

from .registry import registry as default_registry
from .widgets import ChoiceWidget, MultipleChoiceWidget
from .exceptions import AutocompleteChoicesMustBeQuerySet

__all__ = ['FieldBase', 'ChoiceField', 'MultipleChoiceField',
    'ModelChoiceField', 'ModelMultipleChoiceField', 'GenericModelChoiceField',
    'GenericModelMultipleChoiceField']


class FieldBase(object):
    default_error_messages = {
        'invalid': '%(autocomplete)s cannot validate %(value)s',
    }

    def __init__(self, autocomplete=None, registry=None, widget=None,
            widget_js_attributes=None, autocomplete_js_attributes=None,
            extra_context=None, *args, **kwargs):

        self.autocomplete = self.get_autocomplete(autocomplete, registry,
                widget)

        widget = widget or self.widget
        if isinstance(widget, type):
            widget = widget(autocomplete, widget_js_attributes,
                    autocomplete_js_attributes, extra_context)
        kwargs['widget'] = widget

        # Does the subclass have ModelChoiceField or ModelMultipleChoiceField
        # as a base class?
        parents = type(self).__mro__
        if (forms.ModelChoiceField in parents or
                forms.ModelMultipleChoiceField in parents):

            if not isinstance(self.autocomplete.choices, QuerySet):
                raise AutocompleteChoicesMustBeQuerySet(self.autocomplete)
            kwargs['queryset'] = self.autocomplete.choices

        super(FieldBase, self).__init__(*args, **kwargs)

    def get_autocomplete(self, autocomplete, registry, widget):
        if widget:
            # BC: maybe defining the autocomplete as a widget argument ?
            autocomplete = getattr(widget, 'autocomplete', None)

        registry = registry or default_registry
        return registry.get_autocomplete_from_arg(autocomplete)

    def validate(self, value):
        """
        Wrap around forms.Field and Autocomplete.validate_values().

        Field.validate_values() handles the required option.
        """
        forms.Field.validate(self, value)

        # FIXME: we might actually want to change the Autocomplete API to
        # support python values instead of raw values, that would probably be
        # more performant.
        values = self.prepare_value(value)

        if value and not self.autocomplete(values=values).validate_values():
            error_params = {
                'autocomplete': self.autocomplete.__name__, 'value': value
            }

            raise forms.ValidationError(self.error_messages['invalid'],
                                        code='invalid', params=error_params)


class ChoiceField(FieldBase, forms.ChoiceField):
    widget = ChoiceWidget

    def __init__(self, autocomplete=None, registry=None, widget=None,
            widget_js_attributes=None, autocomplete_js_attributes=None,
            extra_context=None, *args, **kwargs):

        kwargs.update({'choices':
            self.get_choices(autocomplete, registry, widget)})

        super(ChoiceField, self).__init__(autocomplete, registry, widget,
                widget_js_attributes, autocomplete_js_attributes,
                extra_context, *args, **kwargs)

    def get_choices(self, autocomplete, registry, widget):
        a = self.get_autocomplete(autocomplete, registry, widget)()
        return ((a.choice_value(c), a.choice_label(c)) for c in a.choices)


class MultipleChoiceField(ChoiceField, forms.MultipleChoiceField):
    widget = MultipleChoiceWidget


class ModelChoiceFieldBase(FieldBase):
    def _get_queryset(self):
        return self._queryset

    def _set_queryset(self, queryset):
        self._queryset = queryset
        self.widget.choices = self.choices

        # Also update autocomplete choices
        self.autocomplete.choices = queryset

    queryset = property(_get_queryset, _set_queryset)


class ModelChoiceField(ModelChoiceFieldBase, forms.ModelChoiceField):
    widget = ChoiceWidget


class ModelMultipleChoiceField(ModelChoiceFieldBase,
        forms.ModelMultipleChoiceField):
    widget = MultipleChoiceWidget


class GenericModelChoiceField(ModelChoiceFieldBase, forms.Field):
    """
    Simple form field that converts strings to models.
    """
    widget = ChoiceWidget

    def prepare_value(self, value):
        """
        Given a model instance as value, with content type id of 3 and pk of 5,
        return such a string '3-5'.
        """
        from django.contrib.contenttypes.models import ContentType

        if isinstance(value, six.string_types):
            # Apparently there's a bug in django, that causes a python value to
            # be passed here. This ONLY happens when in an inline ....
            return value
        elif isinstance(value, models.Model):
            content_type = ContentType.objects.get_for_model(
                value, for_concrete_model=False)
            return '%s-%s' % (content_type.pk, value.pk)

    def to_python(self, value):
        """
        Given a string like '3-5', return the model of content type id 3 and pk
        5.
        """
        from django.contrib.contenttypes.models import ContentType

        if not value:
            return value

        content_type_id, object_id = value.split('-', 1)
        try:
            content_type = ContentType.objects.get_for_id(content_type_id)
        except ContentType.DoesNotExist:
            raise forms.ValidationError('Wrong content type')
        else:
            model = content_type.model_class()

        try:
            return model.objects.get(pk=object_id)
        except model.DoesNotExist:
            raise forms.ValidationError('Wrong object id')


class GenericModelMultipleChoiceField(GenericModelChoiceField):
    """
    Simple form field that converts strings to models.
    """
    widget = MultipleChoiceWidget

    def prepare_value(self, value):
        return [super(GenericModelMultipleChoiceField, self
            ).prepare_value(v) for v in value]

    def to_python(self, value):
        return [super(GenericModelMultipleChoiceField, self).to_python(v)
            for v in value]



try:
    from unittest import mock
except ImportError:  # python2
    import mock

import autocomplete_light.shortcuts as autocomplete_light
from autocomplete_light.compat import urls, url
from django import test

try:
    from django.test import override_settings
except ImportError:
    override_settings = None

urlpatterns = urls([
    url(r'nokwarg/$', mock.Mock, name='test_nokwarg'),
    url(r'onekwarg/(?P<param>\w+)/$', mock.Mock, name='test_onekwarg'),
])


class GetAddAnotherUrlTestCase(test.TestCase):

    def generate_url(self, name, kwargs=None):
        class TestAutocomplete(autocomplete_light.AutocompleteBase):
            add_another_url_name = name
            add_another_url_kwargs = kwargs

        return TestAutocomplete().get_add_another_url()

    def test_no_kwargs(self):
        self.assertEquals(self.generate_url('test_nokwarg'),
                          '/nokwarg/?_popup=1')

    def test_with_kwargs(self):
        self.assertEquals(self.generate_url('test_onekwarg', {'param': 'bar'}),
                          '/onekwarg/bar/?_popup=1')

if override_settings:
    GetAddAnotherUrlTestCase = override_settings(ROOT_URLCONF='autocomplete_light.tests.autocomplete.test_get_add_another_url')(GetAddAnotherUrlTestCase)
else:
    GetAddAnotherUrlTestCase.urls = 'autocomplete_light.tests.autocomplete.test_get_add_another_url'



import autocomplete_light.shortcuts as autocomplete_light
from django import forms

from .models import Dummy


class DummyForm(autocomplete_light.ModelForm):
    class Media:
        """
        We're currently using Media here, but that forced to move the
        javascript from the footer to the extrahead block ...

        So that example might change when this situation annoys someone a lot.
        """
        js = ('dependant_autocomplete.js',)

    class Meta:
        model = Dummy
        exclude = []



import autocomplete_light.shortcuts as autocomplete_light

from .models import *

models = [OtoModel, FkModel, MtmModel, GfkModel]

try:
    import genericm2m
except ImportError:
    pass
else:
    models.append(GmtmModel)

for model in models:
    autocomplete_light.register(model)


class A(autocomplete_light.AutocompleteGenericBase):
    choices = [m.objects.all() for m in models]
    search_fields = [['name']] * len(models)

class A2(autocomplete_light.AutocompleteGenericBase):
    choices = [m.objects.all() for m in models]
    search_fields = [['name']] * len(models)
    values = ['1','2', '3' ]
    attrs = {'data-class-defined': 1}

    def __init__(self, *args, **kw):
        self.attrs = self.attrs.copy()
        self.attrs.update({'data-init-defined': 2})

autocomplete_light.register(A)
autocomplete_light.register(A2)


# The autocomplete for class B is used to set up the test case for
# autocomplete_light.tests.widgets.TextWidgetTestCase.test_widget_attrs_copy.
# This bug is triggered only when the autocomplete is registered with a
# widget_attrs dictionary.
class B(autocomplete_light.AutocompleteGenericBase):
    choices = [m.objects.all() for m in models]
    search_fields = [['name']] * len(models)

class B2(autocomplete_light.AutocompleteGenericBase):
    choices = [m.objects.all() for m in models]
    search_fields = [['name']] * len(models)
    attrs = {'data-class-defined': 1}

    def __init__(self, *args, **kw):
        self.attrs = self.attrs.copy()
        self.attrs.update({'data-init-defined': 2})


autocomplete_light.register(B, widget_attrs={'data-widget-maximum-values': 4})
autocomplete_light.register(B2, )


try:
    from taggit.models import Tag
except ImportError:
    pass
else:
    autocomplete_light.register(Tag)



"""
An url to AutocompleteView.

autocomplete_light_autocomplete
    Given a 'autocomplete' argument with the name of the autocomplete, this url
    routes to AutocompleteView.

autocomplete_light_registry
    Renders the autocomplete registry, good for debugging, requires being
    authenticated as superuser.
"""
from .compat import url, urls

from .views import AutocompleteView, RegistryView


urlpatterns = urls([
    url(r'^(?P<autocomplete>[-\w]+)/$',
        AutocompleteView.as_view(),
        name='autocomplete_light_autocomplete'
    ),
    url(r'^$',
        RegistryView.as_view(),
        name='autocomplete_light_registry'
    ),
])



import autocomplete_light.shortcuts as autocomplete_light
from django import http

from .models import OnTheFly


class OnTheFlyAutocomplete(autocomplete_light.AutocompleteModelBase):
    choices = OnTheFly.objects.all()

    def autocomplete_html(self):
        html = super(OnTheFlyAutocomplete, self).autocomplete_html()

        q = self.request.REQUEST.get('q')
        if q and not self.choices.filter(name=q).exists():
            html += '<span data-value="create">Create "{}"</span>'.format(q)
        return html

    def post(self, request, *args, **kwargs):
        return http.HttpResponse(
            OnTheFly.objects.create(name=request.POST['name']).pk
        )
autocomplete_light.register(OnTheFly, OnTheFlyAutocomplete)



from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class OnTheFly(models.Model):
    name = models.CharField(max_length=100)
    other_fly = models.ForeignKey('self', null=True, blank=True)

    def __str__(self):
        return self.name



import autocomplete_light.shortcuts as autocomplete_light


class AppConfigWithRegistryAutocomplete(
    autocomplete_light.AutocompleteListBase):

    choices = ['a', 'b']
autocomplete_light.register(AppConfigWithRegistryAutocomplete)



import autocomplete_light.shortcuts as autocomplete_light

from .models import NonAdminAddAnotherModel

autocomplete_light.register(NonAdminAddAnotherModel,
    add_another_url_name='non_admin_add_another_model_create')



from django.contrib import admin

from .forms import OnTheFlyForm
from .models import OnTheFly


class OnTheFlyAdmin(admin.ModelAdmin):
    form = OnTheFlyForm
admin.site.register(OnTheFly, OnTheFlyAdmin)



import unittest

import autocomplete_light.shortcuts as autocomplete_light
import lxml.html
from django import forms, http, VERSION
from django.contrib.contenttypes.models import ContentType
from django.forms.models import modelform_factory
from django.test import TestCase
from django.utils import translation
from django.utils.encoding import force_text

from ..example_apps.basic.forms import (DjangoCompatMeta, FkModelForm,
                                        GfkModelForm, MtmModelForm,
                                        OtoModelForm)
from ..example_apps.basic.models import FkModel, GfkModel, MtmModel, OtoModel
from ..example_apps.autocomplete_test_case_app.models import NonIntegerPk
from ..example_apps.autocomplete_test_case_app.forms import NonIntegerPkForm

try:
    from unittest import mock
except ImportError:  # python2
    import mock

try:
    from ..example_apps.basic.forms import GmtmModelForm
except ImportError:
    GmtmModelForm = None
    GmtmModel = None
else:
    from ..example_apps.basic.models import GmtmModel

try:
    from ..example_apps.basic.forms import TaggitModelForm
except ImportError:
    TaggitModelForm = None
    TaggitModel = None
else:
    from ..example_apps.basic.models import TaggitModel


@unittest.skipIf(VERSION < (1, 5), 'Django < 1.5')
class TestUnuseableVirtualfield(TestCase):
    def test_modelform_factory(self):
        from ..example_apps.unuseable_virtualfield.models import HasVotes

        class MyForm(autocomplete_light.ModelForm):
            class Meta(DjangoCompatMeta):
                model = HasVotes
        MyForm()


class SelectMultipleHelpTextRemovalMixin(object):
    def test_help_text_removed(self):
        class ModelForm(forms.ModelForm):
            class Meta(DjangoCompatMeta):
                model = MtmModel
        form = ModelForm()
        help_text = force_text(form.fields['relation'].help_text).strip()

        class ModelForm(autocomplete_light.ModelForm):
            class Meta(DjangoCompatMeta):
                model = MtmModel
        form = ModelForm()
        my_help_text = force_text(form.fields['relation'].help_text).strip()

        # If help_text is not empty (which is wasn't before Django 1.8 fixed
        # #9321), test that it's empty in autocomplete_light's ModelForm.
        assert not help_text or help_text not in my_help_text


class SelectMultipleHelpTextRemovalMixinFrTestCase(
        SelectMultipleHelpTextRemovalMixin, TestCase):

    def setUp(self):
        translation.activate('fr_FR')


class BaseModelFormMixin(object):
    def setUp(self):
        self.james = self.model_class.objects.create(name='James')
        self.janis = self.model_class.objects.create(name='Janis')
        self.test_instance = self.james

    def tearDown(self):
        self.model_class.objects.all().delete()


class ModelFormBaseMixin(BaseModelFormMixin):
    widget_class = autocomplete_light.ChoiceWidget

    def get_new_autocomplete_class(self):
        class SpecialAutocomplete(autocomplete_light.AutocompleteModelBase):
            model = self.model_class
        return SpecialAutocomplete

    def form_value(self, model):
        return 'relation=%s' % model.pk

    def field_value(self, model):
        return getattr(model, 'relation')

    def assertExpectedFormField(self, name='relation'):
        self.assertInForm(name)

        if self.__class__.__name__ != 'TaggitModelFormTestCase':
            # django-taggit enforces verbose_name=_('Tags')
            # bug reported at:
            # https://github.com/alex/django-taggit/issues/177
            self.assertEqual(force_text(self.form[name].label), name.capitalize())

        self.assertTrue(isinstance(self.form.fields[name],
            self.field_class))
        self.assertTrue(isinstance(self.form.fields[name].widget,
            self.widget_class))
        self.assertEqual(self.form.fields[name].autocomplete.__name__,
                self.autocomplete_name)

    def assertInForm(self, name):
        self.assertIn(name, self.form.fields)

    def assertNotInForm(self, name):
        self.assertNotIn(name, self.form.fields)

    def assertIsAutocomplete(self, name):
        self.assertIsInstance(self.form.fields[name],
                autocomplete_light.FieldBase)

    def assertNotIsAutocomplete(self, name):
        self.assertNotIsInstance(self.form.fields[name],
                autocomplete_light.FieldBase)

    def test_appropriate_field_on_modelform(self):
        self.form = self.model_form_class()

        self.assertExpectedFormField()
        self.assertIsAutocomplete('noise')

    def test_appropriate_field_with_modelformfactory(self):
        form_class = modelform_factory(self.model_class,
                form=self.model_form_class, exclude=[])
        self.form = form_class()

        self.assertExpectedFormField()
        self.assertIsAutocomplete('noise')

    @unittest.skipUnless(VERSION >= (1, 6), 'Django >= 1.6')
    def test_appropriate_field_on_modelform_with_all(self):
        class ModelForm(autocomplete_light.ModelForm):
            class Meta:
                model = self.model_class
                fields = '__all__'
        self.form = ModelForm()

        self.assertExpectedFormField()
        self.assertIsAutocomplete('noise')

    def test_appropriate_field_on_modelform_with_formfield_callback(self):
        # This tests what django admin does
        def cb(f, **kwargs):
            return f.formfield(**kwargs)

        form_class = modelform_factory(self.model_class,
                form=self.model_form_class, formfield_callback=cb,
                exclude=[])
        self.form = form_class()

        self.assertExpectedFormField()
        self.assertIsAutocomplete('noise')
        self.assertInForm('name')

    def test_widget_override(self):
        class ModelForm(autocomplete_light.ModelForm):
            class Meta(DjangoCompatMeta):
                model = self.model_class
                widgets = {'relation': self.widget_class(
                    autocomplete=self.autocomplete_name,
                    widget_attrs={'class': 'test-class', 'data-foo': 'bar'}
                )}

        self.form = ModelForm()

        et = lxml.html.fromstring(self.form.as_p())
        attrib = et.cssselect('.autocomplete-light-widget.relation')[0].attrib
        self.assertEquals(attrib['data-foo'], 'bar')
        self.assertIn('test-class', attrib['class'])

    def test_meta_exclude_name(self):
        class ModelForm(autocomplete_light.ModelForm):
            class Meta:
                model = self.model_class
                exclude = ('name',)

        self.form = ModelForm()

        self.assertExpectedFormField()
        self.assertNotInForm('name')
        self.assertIsAutocomplete('noise')

    def test_meta_exclude_relation(self):
        class ModelForm(autocomplete_light.ModelForm):
            class Meta:
                model = self.model_class
                exclude = ['relation']

        self.form = ModelForm()

        self.assertInForm('name')
        self.assertIsAutocomplete('noise')
        self.assertNotInForm('relation')

    def test_meta_fields_name(self):
        class ModelForm(autocomplete_light.ModelForm):
            class Meta:
                model = self.model_class
                fields = ['name']

        self.form = ModelForm()

        self.assertInForm('name')
        self.assertNotInForm('noise')
        self.assertNotInForm('relation')

    def test_meta_fields_relation(self):
        class ModelForm(autocomplete_light.ModelForm):
            class Meta:
                model = self.model_class
                fields = ['relation']

        self.form = ModelForm()

        self.assertExpectedFormField()
        self.assertNotInForm('name')
        self.assertNotInForm('noise')

    def test_meta_autocomplete_fields(self):
        class ModelForm(autocomplete_light.ModelForm):
            class Meta(DjangoCompatMeta):
                model = self.model_class
                autocomplete_fields = ['relation']

        self.form = ModelForm()

        self.assertExpectedFormField()
        self.assertNotIsAutocomplete('noise')
        self.assertInForm('name')

    def test_meta_autocomplete_exclude(self):
        class ModelForm(autocomplete_light.ModelForm):
            class Meta(DjangoCompatMeta):
                model = self.model_class
                autocomplete_exclude = ['relation']

        self.form = ModelForm()

        self.assertInForm('name')
        self.assertNotIsAutocomplete('relation')
        self.assertIsAutocomplete('noise')

    def test_meta_autocomplete_names(self):
        SpecialAutocomplete = self.get_new_autocomplete_class()
        autocomplete_light.register(SpecialAutocomplete)

        class ModelForm(autocomplete_light.ModelForm):
            class Meta(DjangoCompatMeta):
                model = self.model_class
                autocomplete_names = {
                    'relation': 'SpecialAutocomplete'
                }

        self.form = ModelForm()

        self.assertInForm('name')
        self.assertIsAutocomplete('relation')
        self.assertIsAutocomplete('noise')

        self.assertTrue(issubclass(self.form.fields['relation'].autocomplete,
                                   SpecialAutocomplete))
        autocomplete_light.registry.unregister('SpecialAutocomplete')


    def test_modelform_factory(self):
        self.form = autocomplete_light.modelform_factory(self.model_class,
                exclude=[])()

        self.assertExpectedFormField()

    @unittest.skipUnless((1, 8) >= VERSION >= (1, 6), 'Django >= 1.6')
    def test_modelform_factory_does_not_warn(self):
        # fix for #257
        with mock.patch('warnings.warn') as warn:
            self.form = autocomplete_light.modelform_factory(self.model_class,
                    fields='__all__')()
            self.assertEqual(warn.call_count, 0)

        self.assertExpectedFormField()

    def test_modelform_factory_fields_relation(self):
        self.form = autocomplete_light.modelform_factory(self.model_class,
                fields=['relation'])()

        self.assertExpectedFormField()
        self.assertNotInForm('name')
        self.assertNotInForm('noise')

    def test_modelform_factory_exclude_relation(self):
        self.form = autocomplete_light.modelform_factory(self.model_class,
                exclude=['relation'])()

        self.assertNotInForm('relation')
        self.assertInForm('name')
        self.assertIsAutocomplete('noise')

    def test_modelform_factory_autocomplete_fields_relation(self):
        if VERSION < (1, 7):
            fields = None
        else:
            fields = '__all__'

        self.form = autocomplete_light.modelform_factory(self.model_class,
                autocomplete_fields=['relation'], fields=fields)()

        self.assertExpectedFormField()
        self.assertNotIsAutocomplete('noise')
        self.assertInForm('name')

    def test_modelform_factory_autocomplete_exclude_relation(self):
        self.form = autocomplete_light.modelform_factory(self.model_class,
                autocomplete_exclude=['relation'], exclude=[])()

        self.assertNotIsAutocomplete('relation')
        self.assertInForm('name')
        self.assertIsAutocomplete('noise')

    def test_modelform_factory_fields_name(self):
        self.form = autocomplete_light.modelform_factory(self.model_class,
                fields=['name'])()

        self.assertInForm('name')
        self.assertNotInForm('relation')
        self.assertNotInForm('noise')

    def test_modelform_factory_exclude_name(self):
        self.form = autocomplete_light.modelform_factory(self.model_class,
                exclude=['name'])()

        self.assertNotInForm('name')
        self.assertExpectedFormField()
        self.assertIsAutocomplete('noise')

    def test_modelform_factory_autocomplete_names(self):
        SpecialAutocomplete = self.get_new_autocomplete_class()
        autocomplete_light.registry.register(SpecialAutocomplete)

        ModelForm = autocomplete_light.modelform_factory(self.model_class,
            autocomplete_names={'relation': 'SpecialAutocomplete'}, 
            exclude=[])

        self.form = ModelForm()

        self.assertInForm('name')
        self.assertIsAutocomplete('relation')
        self.assertIsAutocomplete('noise')

        self.assertTrue(issubclass(self.form.fields['relation'].autocomplete,
                                   SpecialAutocomplete))

    def test_empty_registry(self):
        registry = autocomplete_light.AutocompleteRegistry()

        class ModelForm(autocomplete_light.ModelForm):
            relation = self.field_class(registry=registry,
                autocomplete=registry.register(self.model_class))
            relation2 = self.field_class(registry=registry,
                autocomplete=registry.register(self.model_class))

            class Meta(DjangoCompatMeta):
                model = self.model_class

        self.form = ModelForm()

        self.assertExpectedFormField()
        self.assertExpectedFormField('relation2')

    def test_create_with_relation(self):
        form = self.model_form_class(http.QueryDict(
            'name=test&%s' % self.form_value(self.janis)))

        self.assertTrue(form.is_valid())

        result = form.save()
        self.assertEqual(self.field_value(result), self.janis)

    def test_add_relation(self):
        form = self.model_form_class(http.QueryDict(
            'name=test&%s' % self.form_value(self.janis)),
            instance=self.test_instance)

        self.assertTrue(form.is_valid())

        result = form.save()
        self.assertEqual(self.field_value(result), self.janis)

    def test_meta_in_parent(self):
        class DefaultForm(autocomplete_light.ModelForm):
            class Meta:
                model = self.model_class
                exclude = []

        class MyForm(DefaultForm):
            pass

        self.form = MyForm()

        self.assertExpectedFormField()
        self.assertIsAutocomplete('noise')

    def test_modelform_without_model(self):
        class DefaultForm(autocomplete_light.ModelForm):
            class Meta:
                pass

        class MyForm(DefaultForm):
            class Meta:
                model = self.model_class
                exclude = []

        self.form = MyForm()

        self.assertExpectedFormField()
        self.assertIsAutocomplete('noise')


class GenericModelFormMixin(object):
    autocomplete_name = 'A'


    def get_new_autocomplete_class(self):
        class SpecialAutocomplete(autocomplete_light.AutocompleteGenericBase):
            choices = autocomplete_light.registry[self.autocomplete_name].choices
            search_fields = autocomplete_light.registry[self.autocomplete_name].search_fields
        return SpecialAutocomplete

    def test_meta_autocomplete_exclude(self):
        class ModelForm(autocomplete_light.ModelForm):
            class Meta(DjangoCompatMeta):
                model = self.model_class
                autocomplete_exclude = ['relation']

        self.form = ModelForm()

        self.assertNotInForm('relation')
        self.assertInForm('name')
        self.assertIsAutocomplete('noise')

    def test_modelform_factory_autocomplete_exclude_relation(self):
        self.form = autocomplete_light.modelform_factory(self.model_class,
                autocomplete_exclude=['relation'], exclude=[])()

        self.assertNotInForm('relation')
        self.assertInForm('name')
        self.assertIsAutocomplete('noise')

    def test_empty_registry(self):
        registry = autocomplete_light.AutocompleteRegistry()

        class ModelForm(autocomplete_light.ModelForm):
            relation = self.field_class(registry=registry,
                autocomplete=registry.register(autocomplete_light.AutocompleteGenericBase,
                    choices=[self.model_class.objects.all()],
                    search_fields=['name']))

            class Meta(DjangoCompatMeta):
                model = self.model_class

        self.form = ModelForm()

        self.assertExpectedFormField()
        self.assertInForm('name')
        self.assertIsAutocomplete('noise')

    def form_value(self, model):
        return 'relation=%s-%s' % (ContentType.objects.get_for_model(model).pk, model.pk)


class MultipleRelationMixin(ModelFormBaseMixin):
    widget_class = autocomplete_light.MultipleChoiceWidget

    def field_value(self, model):
        return super(MultipleRelationMixin, self).field_value(model).all()[0]


class FkModelFormTestCase(ModelFormBaseMixin, TestCase):
    model_class = FkModel
    model_form_class = FkModelForm
    field_class = autocomplete_light.ModelChoiceField
    autocomplete_name = 'FkModelAutocomplete'


class OtoModelFormTestCase(ModelFormBaseMixin, TestCase):
    model_class = OtoModel
    model_form_class = OtoModelForm
    field_class = autocomplete_light.ModelChoiceField
    autocomplete_name = 'OtoModelAutocomplete'


class GfkModelFormTestCase(GenericModelFormMixin,
        ModelFormBaseMixin, TestCase):
    model_class = GfkModel
    model_form_class = GfkModelForm
    field_class = autocomplete_light.GenericModelChoiceField


class MtmModelFormTestCase(MultipleRelationMixin, ModelFormBaseMixin,
        TestCase):
    model_class = MtmModel
    model_form_class = MtmModelForm
    field_class = autocomplete_light.ModelMultipleChoiceField
    autocomplete_name = 'MtmModelAutocomplete'


@unittest.skipIf(TaggitModelForm is None, "taggit is not available.")
class TaggitModelFormTestCase(ModelFormBaseMixin, TestCase):
    model_class = TaggitModel
    model_form_class = TaggitModelForm
    field_class = autocomplete_light.TaggitField
    widget_class = autocomplete_light.TaggitWidget
    autocomplete_name = 'TagAutocomplete'

    def setUp(self):
        self.james = 'james'
        self.janis = 'janis'
        self.test_instance = self.model_class.objects.create(name='test')

    def form_value(self, model):
        return 'relation=%s' % model

    def field_value(self, model):
        return model.relation.all().values_list('name', flat=True)[0]

    def test_empty_registry(self):
        pass

    def test_widget_override(self):
        class ModelForm(autocomplete_light.ModelForm):
            class Meta(DjangoCompatMeta):
                model = self.model_class
                widgets = {'relation': self.widget_class(attrs={
                    'class': 'test-class', 'data-foo': 'bar'})}

        self.form = ModelForm()

        et = lxml.html.fromstring(self.form.as_p())
        attrib = et.cssselect('input[name=relation].autocomplete')[0].attrib
        self.assertEquals(attrib['data-foo'], 'bar')
        self.assertIn('test-class', attrib['class'])


@unittest.skipIf(GmtmModelForm is None, "genericm2m is not available.")
class GmtmModelFormTestCase(MultipleRelationMixin,
        GenericModelFormMixin,
        ModelFormBaseMixin, TestCase):
    model_class = GmtmModel
    model_form_class = GmtmModelForm
    field_class = autocomplete_light.GenericModelMultipleChoiceField

    def field_value(self, model):
        return getattr(model, 'relation').all().generic_objects()[0]


class NonIntegerPkTestCase(ModelFormBaseMixin, TestCase):
    model_class = NonIntegerPk
    model_form_class = NonIntegerPkForm
    field_class = autocomplete_light.ModelChoiceField
    autocomplete_name = 'NonIntegerPkAutocomplete'



from django import VERSION

if VERSION < (1, 5):
    from django.conf.urls.defaults import patterns, url  # noqa
elif VERSION < (1, 8):
    from django.conf.urls import patterns, url  # noqa
else:
    from django.conf.urls import url  # noqa
    patterns = None


try:
    # Django 1.7 or over use the new application loading system
    from django.apps import apps
    get_model = apps.get_model
except ImportError:
    from django.db.models.loading import get_model

try:
    from django.utils.module_loading import import_string
except ImportError:
    from importlib import import_module
    from django.utils import six
    import sys

    def import_string(dotted_path):
        """
        Import a dotted module path and return the attribute/class designated
        by the last name in the path. Raise ImportError if the import failed.
        Backported from Django 1.8
        """
        try:
            module_path, class_name = dotted_path.rsplit('.', 1)
        except ValueError:
            msg = "%s doesn't look like a module path" % dotted_path
            six.reraise(ImportError, ImportError(msg), sys.exc_info()[2])

        module = import_module(module_path)

        try:
            return getattr(module, class_name)
        except AttributeError:
            msg = 'Module "%s" does not define a "%s" attribute/class' % (
                module_path, class_name)
            six.reraise(ImportError, ImportError(msg), sys.exc_info()[2])


def urls(urls):
    if patterns:
        return patterns('', *urls)
    return urls

__all__ = (
    'get_model',
    'import_string',
    'urls',
    'url',
    'patterns'
)



class AutocompleteLightException(Exception):
    """ Base Exception for all exceptions of this module. """
    pass


class AutocompleteNotRegistered(AutocompleteLightException):
    """ Attemps to drive the user to debug his registry. """
    def __init__(self, name, registry):
        if registry.keys():
            msg = '%s not registered, you have registered: %s' % (name,
                    list(registry.keys()))
        else:
            msg = '%s not registered (registry is empty)' % name

        super(AutocompleteNotRegistered, self).__init__(msg)


class AutocompleteArgNotUnderstood(AutocompleteLightException):
    """
    Raised by AutocompleteRegistry.get_autocomplete_from_arg when it cannot
    understand the argument.
    """
    def __init__(self, arg, registry):
        msg = '%s not understod by get_autocomplete_from_arg()' % arg
        super(AutocompleteArgNotUnderstood, self).__init__(msg)


class NoGenericAutocompleteRegistered(AutocompleteLightException):
    """
    Raised by AutocompleteRegistry.autocomplete_for_generic when no generic
    autocomplete has been registered.
    """
    def __init__(self, registry):
        msg = 'No generic autocomplete was registered.'
        super(NoGenericAutocompleteRegistered, self).__init__(msg)


class AutocompleteChoicesMustBeQuerySet(AutocompleteLightException):
    """
    Raised by FieldBase constructor when used in conjunction with
    Model(Multiple)ChoiceField given an Autocomplete class which doesn't have a
    QuerySet for the choices attribute.
    """
    def __init__(self, autocomplete):
        msg = ('%s.choices must be a QuerySet to support ModelChoiceField' %
            autocomplete)
        super(AutocompleteChoicesMustBeQuerySet, self).__init__(msg)


class NonDjangoModelSubclassException(AutocompleteLightException):
    pass



from django import VERSION
from django.views import generic

from autocomplete_light.compat import url, urls
from .forms import AutocompleteItemForm, DjangoItemForm
from .models import Item

urlpatterns = urls([
    url(r'autocomplete/(?P<pk>\d+)/', generic.UpdateView.as_view(
        form_class=AutocompleteItemForm,
        queryset=Item.objects.filter(private=False))),
    url(r'django/(?P<pk>\d+)/', generic.UpdateView.as_view(
        form_class=DjangoItemForm,
        queryset=Item.objects.filter(private=False))),
])



from __future__ import unicode_literals

import six

from django.db import models
from .compat import get_model, import_string

from .exceptions import (AutocompleteArgNotUnderstood,
                         AutocompleteNotRegistered,
                         NoGenericAutocompleteRegistered,
                         NonDjangoModelSubclassException)

"""
The registry module provides tools to maintain a registry of autocompletes.

The first thing that should happen when django starts is registration of
autocompletes. It should happen first, because autocompletes are required for
widgets. And autocomplete widgets are required for forms. And forms are
required for ModelAdmin.

It looks like this:

- in ``yourapp/autocomplete_light_registry.py``, register your autocompletes
  with :py:func:`autocomplete_light.register() <register>`,
- in ``urls.py``, call :py:func:`autocomplete_light.autodiscover()
  <autodiscover>` **before** :py:func:`admin.autodiscover()`.

.. py:data:: registry

    Module-level instance of :py:class:`AutocompleteRegistry`.
"""

try:
    from django.utils.module_loading import autodiscover_modules
except ImportError:
    autodiscover_modules = None

__all__ = ('AutocompleteRegistry', 'registry', 'register', 'autodiscover')


class AutocompleteRegistry(dict):
    """
    AutocompleteRegistry is a dict of ``AutocompleteName: AutocompleteClass``
    with some shortcuts to handle a registry of autocompletes.

    .. py:attribute:: autocomplete_model_base

        The default model autocomplete class to use when registering a Model
        without Autocomplete class. Default is
        :py:class:`~.autocomplete.AutocompleteModelBase`. You can override
        it just before calling autodiscover() in urls.py as such::

            import autocomplete_light.shortcuts as al
            al.registry.autocomplete_model_base = al.AutocompleteModelTemplate
            al.autodiscover()
    """

    def __init__(self, autocomplete_model_base=None):
        """
        You can pass a custom base autocomplete which will be set to
        :py:attr:`autocomplete_model_base` when instanciating an
        AutocompleteRegistry.
        """
        self._models = {}
        self.default_generic = None
        self.autocomplete_model_base = autocomplete_model_base

    def autocomplete_for_model(self, model):
        """
        Return the default autocomplete class for a given model or None.
        """
        try:
            return self._models[model]
        except KeyError:
            return

    def autocomplete_for_generic(self):
        """ Return the default generic autocomplete. """
        if self.default_generic is None:
            raise NoGenericAutocompleteRegistered(self)

        return self.default_generic

    def unregister(self, name):
        """ Unregister a autocomplete given a name. """
        autocomplete = self[name]
        del self[name]

        try:
            if self._models[autocomplete.choices.model].name == name:
                del self._models[autocomplete.choices.model]
        except AttributeError:
            pass

    @classmethod
    def extract_args(cls, *args):
        """
        Takes any arguments like a model and an autocomplete, or just one of
        those, in any order, and return a model and autocomplete.
        """
        model = None
        autocomplete = None

        for arg in args:
            if issubclass(arg, models.Model):
                model = arg
            else:
                autocomplete = arg

        return [model, autocomplete]

    def register(self, *args, **kwargs):
        """
        Register an autocomplete.

        Two unordered arguments are accepted, at least one should be passed:

        - a model if not a generic autocomplete,
        - an autocomplete class if necessary, else one will be generated.

        'name' is also an acceptable keyword argument, that can be used to
        override the default autocomplete name which is the class name by
        default, which could cause name conflicts in some rare cases.

        In addition, keyword arguments will be set as class attributes.

        For thread safety reasons, a copy of the autocomplete class is stored
        in the registry.
        """
        assert len(args) <= 2, 'register takes at most 2 args'
        assert len(args) > 0, 'register takes at least 1 arg'

        processed_args = []

        for arg in args:
            if isinstance(arg, six.string_types):
                parts = arg.split('.')
                if len(parts) == 2:
                    # if 'app_name.ModelName'
                    app_label = parts[0]
                    model_name = parts[-1]
                    model = get_model(app_label=app_label, model_name=model_name)  # noqa
                    processed_args.append(model)
                elif len(parts) > 2:
                    # if 'full.path.to.Model'
                    model = import_string(arg)
                    if not issubclass(model, models.Model):
                        raise NonDjangoModelSubclassException('%s not is subclass of django Model'  # noqa
                                                              % model.__name__)
                    processed_args.append(model)
                else:
                    processed_args.append(arg)
            else:
                processed_args.append(arg)

        model, autocomplete = self.__class__.extract_args(*processed_args)

        # If calling register(YourBaseAutocomplete, YourModel) then you want
        # the autocomplete name to be YourModelYourBaseAutocomplete, but if
        # calling register(YourBaseAutocomplete) then name should be
        # YourBaseAutocomplete
        derivate_name = model and autocomplete

        if autocomplete and not model:
            try:
                model = autocomplete.choices.model
            except AttributeError:
                model = getattr(autocomplete, 'model', None)

        if model:
            autocomplete = self._register_model_autocomplete(model,
                                                             autocomplete,
                                                             derivate_name,
                                                             **kwargs)
        else:
            name = kwargs.get('name', autocomplete.__name__)
            autocomplete = type(str(name), (autocomplete,), kwargs)
            self._register_autocomplete(autocomplete)

        return autocomplete

    def _register_model_autocomplete(self, model, autocomplete=None,
                                     derivate_name=False, name=None, **kwargs):
        if name is not None:
            pass

        elif autocomplete is not None:
            if autocomplete.__name__.find(model.__name__) == 0:
                derivate_name = False

            if derivate_name:
                name = '%s%s' % (model.__name__, autocomplete.__name__)
            else:
                name = autocomplete.__name__
        else:
            name = '%sAutocomplete' % model.__name__

        if autocomplete is None:
            if self.autocomplete_model_base is None:
                from .autocomplete.shortcuts import AutocompleteModelBase
                self.autocomplete_model_base = AutocompleteModelBase
            base = self.autocomplete_model_base
        else:
            base = autocomplete

        if base.choices is None and 'choices' not in kwargs:
            kwargs['choices'] = model._default_manager.all()

        if base.search_fields is None and 'search_fields' not in kwargs:
            try:
                model._meta.get_field('name')
            except:
                raise Exception('Add search_fields kwargs to .register(%s)'
                                % model.__name__)
            else:
                kwargs['search_fields'] = ['name']

        kwargs.update({'model': model})

        autocomplete = type(str(name), (base,), kwargs)

        self._register_autocomplete(autocomplete)

        if model not in self._models.keys():
            self._models[model] = autocomplete

        return autocomplete

    def _register_autocomplete(self, autocomplete):
        """
        Register a autocomplete without model, like a generic autocomplete.
        """
        self[autocomplete.__name__] = autocomplete

        if not getattr(autocomplete, 'model', False):
            if not self.default_generic:
                self.default_generic = autocomplete

    def __getitem__(self, name):
        """
        Return the Autocomplete class registered for this name. If none is
        registered, raise AutocompleteNotRegistered.
        """
        try:
            return super(AutocompleteRegistry, self).__getitem__(name)
        except KeyError:
            raise AutocompleteNotRegistered(name, self)

    def get_autocomplete_from_arg(self, arg=None):
        from .autocomplete.base import AutocompleteInterface
        if isinstance(arg, six.string_types):
            return self[arg]
        elif isinstance(arg, type) and issubclass(arg, models.Model):
            return self.autocomplete_for_model(arg)
        elif isinstance(arg, models.Model):
            return self.autocomplete_for_model(arg.__class__)
        elif isinstance(arg, type) and issubclass(arg, AutocompleteInterface):
            return arg
        elif arg is None:
            return self.default_generic
        else:
            raise AutocompleteArgNotUnderstood(arg, self)


def _autodiscover(registry):
    """See documentation for autodiscover (without the underscore)"""
    import copy
    from django.conf import settings
    from django.utils.importlib import import_module
    from django.utils.module_loading import module_has_submodule

    for app in settings.INSTALLED_APPS:
        mod = import_module(app)

        # Attempt to import the app's admin module.
        try:
            before_import_registry = copy.copy(registry)
            import_module('%s.autocomplete_light_registry' % app)
        except:
            # Reset the model registry to the state before the last import as
            # this import will have to reoccur on the next request and this
            # could raise NotRegistered and AlreadyRegistered exceptions
            # (see #8245).
            registry = before_import_registry

            # Decide whether to bubble up this error. If the app just
            # doesn't have an admin module, we can ignore the error
            # attempting to import it, otherwise we want it to bubble up.
            if module_has_submodule(mod, 'autocomplete_light_registry'):
                raise


registry = AutocompleteRegistry()


def autodiscover():
    """
    Check all apps in INSTALLED_APPS for stuff related to autocomplete_light.

    For each app, autodiscover imports ``app.autocomplete_light_registry`` if
    possing, resulting in execution of :py:func:`register()` statements in that
    module, filling up :py:data:`registry`.

    Consider a standard app called ``cities_light`` with such a structure::

        cities_light/
            __init__.py
            models.py
            urls.py
            views.py
            autocomplete_light_registry.py

    Where autocomplete_light_registry.py contains something like::

        from models import City, Country
        import autocomplete_light.shortcuts as al
        al.register(City)
        al.register(Country)

    When ``autodiscover()`` imports
    ``cities_light.autocomplete_light_registry``, both ``CityAutocomplete`` and
    ``CountryAutocomplete`` will be registered. See
    :py:meth:`AutocompleteRegistry.register()` for details on how these
    autocomplete classes are generated.
    """
    if autodiscover_modules:
        autodiscover_modules('autocomplete_light_registry')
    else:
        _autodiscover(registry)


def register(*args, **kwargs):
    """
    Proxy method :py:meth:`AutocompleteRegistry.register` of the
    :py:data:`registry` module level instance.
    """
    return registry.register(*args, **kwargs)



from django.db import models


class User(models.Model):
    username = models.CharField(max_length=100)
    email = models.EmailField()
    groups = models.ManyToManyField('Group')
    ordering = models.IntegerField(default=0)

    class Meta:
        ordering = ('ordering',)


class Group(models.Model):
    name = models.CharField(max_length=100)


class NonIntegerPk(models.Model):
    name = models.CharField(primary_key=True, max_length=10)
    relation = models.ForeignKey('self', null=True, blank=True)
    noise = models.ForeignKey('basic.FkModel', null=True, blank=True)

    for_inline = models.ForeignKey('self', null=True, blank=True,
                                   related_name='inline')


class CustomIntegerPk(models.Model):
    id = models.IntegerField(primary_key=True)


class SubGroup(Group):
    pass


class CustomSchema(models.Model):
    name = models.CharField(primary_key=True, max_length=10, db_column='bar')

    class Meta:
        db_table = 'foobar'


class Caps(models.Model):
    id = models.IntegerField(primary_key=True, db_column='Id')
    name = models.CharField(max_length=10, db_column='Bar')

    class Meta:
        db_table = 'Caps'



import autocomplete_light.shortcuts as autocomplete_light
from django.contrib import admin

from .models import NonAdminAddAnotherModel


class NonAdminAddAnotherModelAdmin(admin.ModelAdmin):
    form = autocomplete_light.modelform_factory(NonAdminAddAnotherModel,
            fields=('name', 'widgets'))

admin.site.register(NonAdminAddAnotherModel, NonAdminAddAnotherModelAdmin)



from __future__ import unicode_literals

import pytest

import autocomplete_light.shortcuts as autocomplete_light
from django import forms
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from ..example_apps.basic.models import GfkModel
from ..example_apps.security_test.models import Item


class BaseMixin(object):
    GOOD_VALUE = 'b'
    CLEANED_VALUE = 'b'
    BAD_VALUE = 'xx'
    EMPTY_VALUE = None

    class TestAutocomplete(autocomplete_light.AutocompleteListBase):
        choices = ['a', 'b', 'c']

    def test_automatic_field_choices(self):
        test = self.field_class(self.TestAutocomplete)
        self.assertEqual(test.choices, [('a', 'a'), ('b', 'b'), ('c', 'c')])

    def test_validate(self):
        test = self.field_class(self.TestAutocomplete)
        test.validate(self.GOOD_VALUE)

        with self.assertRaises(forms.ValidationError):
            test.validate(self.BAD_VALUE)

    def test_validate_required(self):
        test = self.field_class(self.TestAutocomplete, required=True)

        with pytest.raises(forms.ValidationError):
            test.validate(self.EMPTY_VALUE)

    def test_select_choice(self):
        class TestForm(forms.Form):
            test_field = self.field_class(self.TestAutocomplete)

        form = TestForm({'test_field': self.GOOD_VALUE})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['test_field'], self.CLEANED_VALUE)


class ChoiceFieldTestCase(BaseMixin, TestCase):
    field_class = autocomplete_light.ChoiceField


class MultipleChoiceFieldTestCase(BaseMixin, TestCase):
    field_class = autocomplete_light.MultipleChoiceField
    GOOD_VALUE = ['b']
    CLEANED_VALUE = ['b']


class ModelChoiceFieldTestCase(BaseMixin, TestCase):
    fixtures = ['security_test.json']
    field_class = autocomplete_light.ModelChoiceField
    GOOD_VALUE = 1
    BAD_VALUE = 2

    def setUp(self):
        self.CLEANED_VALUE = Item.objects.get(pk=self.GOOD_VALUE)

    class TestAutocomplete(autocomplete_light.AutocompleteModelBase):
        choices = Item.objects.filter(private=False)

    def test_automatic_field_choices(self):
        test = self.field_class(self.TestAutocomplete, required=True)
        self.assertEqual(list(test.choices),
                         [('', '---------'), (1, 'public'), (3, 'linked')])

    def test_non_queryset_choices_fails(self):
        class FailAutocomplete(autocomplete_light.AutocompleteModelBase):
            choices = 'bar'

        with self.assertRaises(autocomplete_light.AutocompleteChoicesMustBeQuerySet):
            test = self.field_class(FailAutocomplete)


class ModelMultipleChoiceFieldTestCase(ModelChoiceFieldTestCase):
    field_class = autocomplete_light.ModelMultipleChoiceField
    GOOD_VALUE = [1]
    BAD_VALUE = [2]

    def setUp(self):
        self.CLEANED_VALUE = Item.objects.filter(pk=1)

    def test_automatic_field_choices(self):
        test = self.field_class(self.TestAutocomplete, required=True)
        self.assertEqual(list(test.choices),
                         [(1, 'public'), (3, 'linked')])

    def test_select_choice(self):
        class TestForm(forms.Form):
            test_field = self.field_class(self.TestAutocomplete)

        form = TestForm({'test_field': self.GOOD_VALUE})
        self.assertTrue(form.is_valid())
        self.assertEqual(len(form.cleaned_data['test_field']),
                         len(self.CLEANED_VALUE))
        self.assertEqual(form.cleaned_data['test_field'][0],
                         self.CLEANED_VALUE[0])


class CustomModelChoiceField(forms.ModelChoiceField):
    pass


class CustomModelMultipleChoiceField(forms.ModelMultipleChoiceField):
    pass


class CustomAutocompleteModelChoiceField(autocomplete_light.FieldBase,
                                         CustomModelChoiceField):
    """ Autocomplete form field which inherently, but not directly, inherits
    forms.ModelChoiceField
    """
    widget = autocomplete_light.ChoiceWidget


class CustomAutocompleteModelMultipleChoiceField(autocomplete_light.FieldBase,
                                                 CustomModelMultipleChoiceField):
    """ Autocomplete form field which inherently, but not directly, inherits
    forms.ModelMultipleChoiceField
    """
    widget = autocomplete_light.MultipleChoiceWidget


class CustomModelFieldTestCase(ModelChoiceFieldTestCase):
    """ Regression test for a custom ModelChoiceField
    https://github.com/yourlabs/django-autocomplete-light/issues/379
    """
    field_class = CustomAutocompleteModelChoiceField


class CustomMultipleModelFieldTestCase(ModelMultipleChoiceFieldTestCase):
    """ Regression test for a custom ModelMultipleChoiceField
    https://github.com/yourlabs/django-autocomplete-light/issues/379
    """
    field_class = CustomAutocompleteModelMultipleChoiceField


class GenericModelChoiceFieldTestCase(BaseMixin, TestCase):
    field_class = autocomplete_light.GenericModelChoiceField
    fixtures = ['basic_gfk_gmtm.json']

    class TestAutocomplete(autocomplete_light.AutocompleteGenericBase):
        choices = [GfkModel.objects.all()]

    def setUp(self):
        self.gfk_ct = ContentType.objects.get_for_model(GfkModel)
        self.GOOD_VALUE = '%s-%s' % (self.gfk_ct.pk, 1)
        self.BAD_VALUE = '%s-%s' % (self.gfk_ct.pk, 1234)
        self.CLEANED_VALUE = GfkModel.objects.get(pk=1)

    def test_automatic_field_choices(self):
        pass  # generic model choice field has no choices



class GenericModelMultipleChoiceFieldTestCase(GenericModelChoiceFieldTestCase):
    field_class = autocomplete_light.GenericModelMultipleChoiceField

    def setUp(self):
        self.gfk_ct = ContentType.objects.get_for_model(GfkModel)
        self.GOOD_VALUE = ['%s-%s' % (self.gfk_ct.pk, 1)]
        self.BAD_VALUE = ['%s-%s' % (self.gfk_ct.pk, 1234)]
        self.CLEANED_VALUE = [GfkModel.objects.get(pk=1)]

    def test_automatic_field_choices(self):
        pass  # generic model choice field has no choices



from .base import AutocompleteBase
from .choice_list import AutocompleteChoiceList
from .generic import AutocompleteGeneric
from .list import AutocompleteList
from .model import AutocompleteModel
from .rest_model import AutocompleteRestModel
from .template import AutocompleteTemplate


class AutocompleteListBase(AutocompleteList, AutocompleteBase):
    """
    Use :py:class:`~base.AutocompleteBase` for rendering logic and
    :py:class:`~list.AutocompleteList` for business logic.
    """
    pass


class AutocompleteListTemplate(AutocompleteList, AutocompleteTemplate):
    """
    Use :py:class:`~template.AutocompleteTemplate` for rendering logic and
    :py:class:`~list.AutocompleteList` for business logic.
    """
    pass


class AutocompleteChoiceListBase(AutocompleteChoiceList, AutocompleteBase):
    """
    Use :py:class:`~base.AutocompleteBase` for rendering logic and
    :py:class:`~choice_list.AutocompleteChoiceList` for
    business logic.
    """
    pass


class AutocompleteChoiceListTemplate(AutocompleteChoiceList,
        AutocompleteTemplate):
    """
    Use :py:class:`~template.AutocompleteTemplate` for rendering logic and
    :py:class:`~choice_list.AutocompleteChoiceList` for business logic.
    """
    pass


class AutocompleteModelBase(AutocompleteModel, AutocompleteBase):
    """
    Use :py:class:`~base.AutocompleteBase` for rendering logic and
    :py:class:`~model.AutocompleteModel` for business logic.
    """
    pass


class AutocompleteModelTemplate(AutocompleteModel, AutocompleteTemplate):
    """
    Use :py:class:`~template.AutocompleteTemplate` for rendering logic and
    :py:class:`~model.AutocompleteModel` for business logic.

    It also sets a default :py:attr:`choice_template`.
    """
    choice_template = 'autocomplete_light/model_template/choice.html'


class AutocompleteGenericBase(AutocompleteGeneric, AutocompleteBase):
    """
    Use :py:class:`~base.AutocompleteBase` for rendering logic and
    :py:class:`~generic.AutocompleteGeneric` for business logic.
    """
    pass


class AutocompleteGenericTemplate(AutocompleteGeneric, AutocompleteTemplate):
    """
    Use :py:class:`~template.AutocompleteTemplate` for rendering logic and
    :py:class:`~generic.AutocompleteGeneric` for business logic.
    """
    pass


class AutocompleteRestModelBase(AutocompleteRestModel, AutocompleteBase):
    """
    Use :py:class:`~base.AutocompleteBase` for rendering logic and
    :py:class:`~rest_model.AutocompleteRestModel` for business logic.
    """
    pass


class AutocompleteRestModelTemplate(AutocompleteRestModel,
                                    AutocompleteTemplate):
    """
    Use :py:class:`~template.AutocompleteTemplate` for rendering logic and
    :py:class:`~rest_model.AutocompleteRestModel` for business logic.
    """
    pass



from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class Item(models.Model):
    name = models.CharField(max_length=100)
    items = models.ManyToManyField('self', blank=True)
    private = models.BooleanField(default=True)

    def __str__(self):
        return self.name



from django.apps import AppConfig


class AutocompleteLightConfig(AppConfig):
    name = 'autocomplete_light'

    def ready(self):
        from autocomplete_light.registry import autodiscover
        autodiscover()



from collections import defaultdict
from functools import wraps

import django
import six
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.db import IntegrityError, models, transaction
from django.db.models import Count
from django.db.models.query import QuerySet
from django.utils.translation import ugettext_lazy as _

# Django 1.5 add support for custom auth user model
if django.VERSION >= (1, 5):
    AUTH_USER_MODEL = settings.AUTH_USER_MODEL
else:
    AUTH_USER_MODEL = 'auth.User'

try:
    atomic = transaction.atomic
except AttributeError:
    from contextlib import contextmanager

    @contextmanager
    def atomic(using=None):
        sid = transaction.savepoint(using=using)
        try:
            yield
        except IntegrityError:
            transaction.savepoint_rollback(sid, using=using)
            raise
        else:
            transaction.savepoint_commit(sid, using=using)


try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey


class VoteManger(models.Manager):
    def filter(self, *args, **kwargs):
        if kwargs.has_key('content_object'):
            content_object = kwargs.pop('content_object')
            content_type = ContentType.objects.get_for_model(content_object)
            kwargs.update({
                    'content_type':content_type,
                    'object_id':content_object.pk
                    })
        return super(VoteManger, self).filter(*args, **kwargs)
    
class Vote(models.Model):
    user = models.ForeignKey(AUTH_USER_MODEL)
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey()
    create_at = models.DateTimeField(auto_now_add=True)

    objects = VoteManger()
    
    class Meta:
        unique_together = ('user', 'content_type', 'object_id')
        app_label = 'unuseable_virtualfield'

    @classmethod
    def votes_for(cls, model, instance=None):
        ct = ContentType.objects.get_for_model(model)
        kwargs = {
            "content_type": ct
        }
        if instance is not None:
            kwargs["object_id"] = instance.pk
            
        return cls.objects.filter(**kwargs)

try:
    from django.contrib.contenttypes.fields import GenericRelation
except ImportError:
    from django.contrib.contenttypes.generic import GenericRelation


def instance_required(func):
    @wraps(func)
    def inner(self, *args, **kwargs):
        if self.instance is None:
            raise TypeError("Can't call %s with a non-instance manager" % func.__name__)
        return func(self, *args, **kwargs)
    return inner


class VotedQuerySet(QuerySet):
    """
    if call votes.annotate with an `user` argument then add `is_voted` to each instance
    """
    
    def __init__(self, model=None, query=None, using=None, user=None):
        self.user = user
        super(VotedQuerySet, self).__init__(model=model, query=query, using=using)
        
    def __iter__(self):
        super(VotedQuerySet, self).__iter__()
        if self.user is None:
            return iter(self._result_cache)

        objects = self._result_cache
        user_id = self.user.id
        contenttype = ContentType.objects.get_for_model(self.model)
        object_ids = [r.id for r in objects]
        
        voted_users = defaultdict(list)
        votes = Vote.objects.filter(content_type=contenttype, object_id__in=object_ids)
        for v in votes:
            voted_users[v.object_id].append(v.user_id)
            
        for r in objects:
            r.is_voted = user_id in voted_users.get(r.id, [])

        self._result_cache = objects
        return iter(objects)
    
    def _clone(self):
        c = super(VotedQuerySet, self)._clone()
        c.user = self.user
        return c
    
class _VotableManager(models.Manager):
    def __init__(self, through, model, instance, field_name='votes'):
        self.through = through
        self.model = model
        self.instance = instance
        self.field_name = field_name

    @instance_required
    def up(self, user):
        self.through(user=user, content_object=self.instance).save()
        
    @instance_required
    def down(self, user):
        self.through.objects.filter(user=user, content_object=self.instance).delete()

    @instance_required
    def exists(self, user):
        return self.through.objects.filter(user=user, content_object=self.instance).exists()

    def count(self):
        return self.through.votes_for(self.model, self.instance).count()

    def annotate(self, queryset=None, user=None, annotation='num_votes', reverse=True):
        order = reverse and '-%s' % annotation or annotation
        kwargs = {annotation:Count('%s__user' % self.field_name)}
        queryset = queryset if queryset is not None else self.model.objects.all()
        queryset = queryset.annotate(**kwargs).order_by(order, '-id')
        return VotedQuerySet(model=queryset.model, query=queryset.query, user=user)
        
class VotableManager(GenericRelation):
    def __init__(self, through=Vote, manager=_VotableManager, **kwargs):
        self.through = through
        self.manager = manager
        kwargs['verbose_name'] = kwargs.get('verbose_name', _('Votes'))
        super(VotableManager, self).__init__(self.through, **kwargs)
        
    def __get__(self, instance, model):
        if instance is not None and instance.pk is None:
            raise ValueError("%s objects need to have a primary key value "
                "before you can access their votes." % model.__name__)
        manager = self.manager(
            through=self.through,
            model=model,
            instance=instance,
            field_name=self.name
        )
        return manager

    def contribute_to_class(self, cls, name):
        super(VotableManager, self).contribute_to_class(cls, name)
        setattr(cls, name, self)



from __future__ import unicode_literals

import atexit
import os
import unittest

from django import VERSION
from selenium import webdriver
from django.core.urlresolvers import reverse
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import ui
from selenium.webdriver.support.ui import Select

try:
    from taggit.models import Tag
except ImportError:
    Tag = None



if VERSION[0] == 1 and VERSION[1] < 7:
    # Patch for travis
    from django.test.testcases import StoppableWSGIServer

    def patient_shutdown(self):
        """
        Stops the serve_forever loop.

        Blocks until the loop has finished. This must be called while
        serve_forever() is running in another thread, or it will
        deadlock.
        """
        self._StoppableWSGIServer__serving = False
        if not self._StoppableWSGIServer__is_shut_down.wait(30):
            raise RuntimeError(
                "Failed to shutdown the live test server in 2 seconds. The "
                "server might be stuck or generating a slow response.")
    StoppableWSGIServer.shutdown = patient_shutdown

    from django.test import LiveServerTestCase
else:
    # LiveServerTestCase doesn't serve static files in 1.7 anymore
    from django.contrib.staticfiles.testing import StaticLiveServerTestCase as LiveServerTestCase


if os.environ.get('TRAVIS', False):
    WAIT_TIME = 30
elif os.environ.get('BUILD_ID', False):  #  Jenkins build server
    WAIT_TIME = 30
else:
    WAIT_TIME = 5


# Global Selenium instance.
class Selenium(object):
    selenium = None

    def __new__(cls):
        if not cls.selenium:
            selenium = getattr(webdriver, os.environ.get('TESTS_WEBDRIVER', 'Firefox'))()
            selenium.implicitly_wait(WAIT_TIME)
            atexit.register(selenium.quit)
            cls.selenium = selenium
        return cls.selenium


class WidgetTestCase(LiveServerTestCase):
    input_name_suffix = '-autocomplete'
    autocomplete_name = 'relation'
    fixtures = ['basic_fk_model_test_case.json', 'test_user.json']
    test_case_setup_done = False

    @classmethod
    def setUpClass(cls):
        if os.environ.get('TESTS_SKIP_LIVESERVER', False):
            raise unittest.SkipTest('TESTS_SKIP_LIVESERVER enabled')

        cls.selenium = Selenium()
        super(WidgetTestCase, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        super(WidgetTestCase, cls).tearDownClass()
        cls.test_case_setup_done = False

    def setUp(self):
        if self.__class__.test_case_setup_done is False:
            self.set_implicit_wait()
            self.setup_test_case()
        self.__class__.test_case_setup_done = True

    def open_url(self, url):
        self.selenium.get('%s%s' % (self.live_server_url, url))

    def send_keys(self, keys, autocomplete_name=None):
        autocomplete_name = autocomplete_name or self.autocomplete_name

        for key in keys:
            self.selenium.find_element_by_css_selector(
                'input[name=%s%s]' % (autocomplete_name,
                    self.input_name_suffix)
                ).send_keys(key)

    def submit(self, name=None):
        selector = 'input[type=submit]'

        if name:
            selector += '[name=%s]' % name

        self.selenium.find_element_by_css_selector(selector).click()

    def login(self):
        self.client.login(username='test', password='test')
        cookie = self.client.cookies['sessionid']
        self.open_url('/admin/')
        self.selenium.add_cookie({'name': 'sessionid', 'value': cookie.value, 'secure': False, 'path': '/'})

    def deck_choice_elements(self, autocomplete_name=None):
        autocomplete_name = autocomplete_name or self.autocomplete_name

        return self.selenium.find_elements_by_css_selector(
            '#id_%s-deck [data-value]' % autocomplete_name)

    def autocomplete(self, autocomplete_name=None):
        autocomplete_name = autocomplete_name or self.autocomplete_name
        return self.selenium.find_element_by_css_selector(
            '.yourlabs-autocomplete[data-input-id="id_'+autocomplete_name+'-autocomplete"]')

    def deck_choices(self, autocomplete_name=None):
        autocomplete_name = autocomplete_name or self.autocomplete_name

        xpath = ''.join([
            '//*[@id="id_%s%s"]/' % (autocomplete_name,
                self.input_name_suffix),
            'preceding-sibling::',
            'span[contains(',
                'concat(" ", normalize-space(@class), " "), ',
                '" deck ")',
            ']/*[@data-value]'])

        return self.selenium.find_elements_by_xpath(xpath)

    def hilighted_choice(self, autocomplete_name=None):
        autocomplete_name = autocomplete_name or self.autocomplete_name
        return self.selenium.find_element_by_css_selector(
            '.yourlabs-autocomplete[data-input-id="id_'+autocomplete_name+'-autocomplete"] .hilight')

    def autocomplete_choices(self, autocomplete_name=None):
        autocomplete_name = autocomplete_name or self.autocomplete_name
        return self.selenium.find_elements_by_css_selector(
            '.yourlabs-autocomplete[data-input-id="id_'+autocomplete_name+'-autocomplete"] [data-value]')

    def input(self, autocomplete_name=None):
        autocomplete_name = autocomplete_name or self.autocomplete_name

        return self.selenium.find_element_by_css_selector(
                'input[name=%s%s]' % (autocomplete_name,
                    self.input_name_suffix))

    def select(self, autocomplete_name=None):
        autocomplete_name = autocomplete_name or self.autocomplete_name

        xpath = ''.join([
            '//*[@id="id_%s%s"]/' % (autocomplete_name,
                self.input_name_suffix),
            'following-sibling::',
            'select'])

        return self.selenium.find_element_by_xpath(xpath)

    def set_implicit_wait(self):
        self.selenium.implicitly_wait(WAIT_TIME)
        self.selenium.set_page_load_timeout(WAIT_TIME)

    def unset_implicit_wait(self):
        self.selenium.implicitly_wait(0)
        self.selenium.set_page_load_timeout(0)

    def select_values(self):
        self.select  # wait for select

        # don't wait for options as there might be none
        self.unset_implicit_wait()

        ret = [o.get_attribute('value') for o in Select(self.select()).options if
                o.is_selected()]

        # restore implicit wait
        self.set_implicit_wait()

        return ret

    def assertSameChoice(self, autocomplete_choice, deck_choice):
        if autocomplete_choice.get_attribute('data-value') != deck_choice.get_attribute('data-value'):
            self.fail('Choices have different data-value')

        if autocomplete_choice.text not in deck_choice.text:
            # deck_choice has an additional span.remove
            self.fail('Choices have different text')

    def assertAutocompleteEmpty(self):
        self.unset_implicit_wait()
        self.assertTrue(len(self.autocomplete_choices()) == 0)
        self.set_implicit_wait()


class ActivateAutocompleteInBlankFormTestCase(WidgetTestCase):
    def setup_test_case(self):
        self.login()
        self.open_url('/admin/basic/fkmodel/add/')
        self.send_keys('ja')

    def test_autocomplete_shows_up(self):
        self.assertTrue(self.autocomplete().is_displayed())

    def test_autocomplete_has_four_choices(self):
        self.assertEqual(4, len(self.autocomplete_choices()))

class XhrPendingTestCase(WidgetTestCase):
    def setup_test_case(self):
        self.login()
        self.open_url('/admin/basic/fkmodel/add/')

    def test_xhr_pending(self):
        self.send_keys('ja')
        self.selenium.find_element_by_css_selector(
            'input[name=%s-autocomplete]' % self.autocomplete_name)
        self.selenium.find_element_by_css_selector(
            'input:not(.xhr-pending)[name=%s-autocomplete]' % self.autocomplete_name)


class SelectChoiceInEmptyFormTestCase(WidgetTestCase):
    def setup_test_case(self):
        self.login()
        self.open_url('/admin/basic/fkmodel/add/')
        self.send_keys('ja')
        self.autocomplete_choices()[1].click()

    def test_autocomplete_disappears(self):
        self.assertFalse(self.autocomplete().is_displayed())

    def test_input_disappears(self):
        self.assertFalse(self.input().is_displayed())

    def test_deck_choice_shows_up(self):
        self.assertEqual(len(self.deck_choices()), 1)

    def test_deck_choice_same_as_selected(self):
        self.assertSameChoice(self.autocomplete_choices()[1], self.deck_choices()[0])

    def test_hidden_select_value(self):
        self.assertEqual(self.select_values(), ['4'])

    def test_admin_change_link_update(self):
        change_link = self.selenium.find_element_by_id('change_id_%s' % self.autocomplete_name)
        href = change_link.get_attribute('href')

        assert href.endswith('%s?_to_field=id&_popup=1' %
                reverse('admin:basic_fkmodel_change', args=(4,)))


@unittest.skipIf(Tag is None, 'django-taggit not installed')
class TextWidgetWithTaggitForm(WidgetTestCase):
    input_name_suffix = ''

    def setup_test_case(self):
        Tag.objects.create(name='foo & bar')
        self.login()
        self.open_url('/admin/basic/taggitmodel/add/')

    def test_ampersand(self):
        self.send_keys('foo & bar')
        ui.WebDriverWait(self.selenium, WAIT_TIME).until(
            lambda x: self.hilighted_choice())

        self.send_keys([Keys.TAB])
        assert 'foo & bar' == self.input().get_attribute('value')


class WidgetInitialStatusInEditForm(WidgetTestCase):
    def setup_test_case(self):
        self.login()
        self.open_url('/admin/basic/fkmodel/1/')

    def test_hidden_select_values(self):
        self.assertEqual(self.select_values(), ['4'])

    def test_input_is_hidden(self):
        self.assertFalse(self.input().is_displayed())


class RemoveChoiceInEditFormTestCase(WidgetTestCase):
    def setup_test_case(self):
        self.login()
        self.open_url('/admin/basic/fkmodel/1/')
        self.deck_choices()[0].find_element_by_css_selector('.remove').click()

    def test_input_shows_up(self):
        self.assertTrue(self.input().is_displayed())

    def test_hidden_select_option_was_unselected(self):
        self.unset_implicit_wait()
        self.assertEqual(self.select_values(), [])
        self.set_implicit_wait()

    def test_element_was_remove_from_deck(self):
        self.unset_implicit_wait()
        self.assertEqual(0, len(self.deck_choices()))
        self.set_implicit_wait()

    def test_admin_change_link_has_no_href(self):
        change_link = self.selenium.find_element_by_id('change_id_%s' % self.autocomplete_name)
        href = change_link.get_attribute('href')
        assert href is None


class KeyboardTestCase(WidgetTestCase):
    def setup_test_case(self):
        self.login()
        self.open_url('/admin/basic/fkmodel/add/')
        self.send_keys('jac')

    def assertHilightedChoiceNmber(self, n):
        self.assertSameChoice(self.hilighted_choice(), self.autocomplete_choices()[n])

    def send_keys_wait_assert_choice_number(self, key, choice):
        old_hilight = self.hilighted_choice()

        self.send_keys([key])
        ui.WebDriverWait(self.selenium, WAIT_TIME).until(
            lambda x: old_hilight != self.hilighted_choice())

        self.assertSameChoice(self.hilighted_choice(), self.autocomplete_choices()[choice])

    def test_00_first_to_second_with_down(self):
        self.send_keys_wait_assert_choice_number(Keys.ARROW_DOWN, 1)

    def test_01_last_to_first_with_down(self):
        self.send_keys_wait_assert_choice_number(Keys.ARROW_DOWN, 0)

    def test_02_first_to_last_with_up(self):
        self.send_keys_wait_assert_choice_number(Keys.ARROW_UP, -1)

    def test_03_last_to_first_with_up(self):
        self.send_keys_wait_assert_choice_number(Keys.ARROW_UP, 0)

    def test_04_tab_to_select_choice(self):
        self.send_keys([Keys.TAB])
        self.assertSameChoice(self.autocomplete_choices()[0], self.deck_choices()[0])
        self.assertEqual(self.select_values(), ['4'])


class InlineBlankTestCase(ActivateAutocompleteInBlankFormTestCase):
    autocomplete_name = 'reverse_for_inline-3-relation'

    def setup_test_case(self):
        self.login()
        self.open_url('/admin/basic/fkmodel/add/')
        self.selenium.find_element_by_css_selector('.add-row a').click()
        self.send_keys('ja')


class InlineSelectChoiceTestCase(SelectChoiceInEmptyFormTestCase):
    autocomplete_name = 'reverse_for_inline-3-relation'

    def setup_test_case(self):
        self.login()
        self.open_url('/admin/basic/fkmodel/add/')
        self.selenium.find_element_by_css_selector('.add-row a').click()
        self.send_keys('ja')
        self.autocomplete_choices()[1].click()



from __future__ import unicode_literals

from collections import OrderedDict

from autocomplete_light.templatetags. \
    autocomplete_light_tags import autocomplete_light_data_attributes
from django.test import TestCase


class DataAttributesTestCase(TestCase):
    def test_without_prefix(self):
        tests = (
            {
                'fixture': {
                    'foo': 'bar',
                },
                'expected': 'data-foo="bar"',
            },
            {
                'fixture': OrderedDict([
                    ('foo', 'bar'),
                    ('test_underscore', 'example'),
                ]),
                'expected': 'data-foo="bar" data-test-underscore="example"',
            },
            {
                'fixture': {
                    'foo': 'bar',
                },
                'prefix': 'autocomplete-',
                'expected': 'data-autocomplete-foo="bar"',
            },
        )

        for test in tests:
            result = autocomplete_light_data_attributes(test['fixture'],
                test.get('prefix', ''))
            self.assertEqual(result, test['expected'],
                'Got %s for %s' % (result, test))



import autocomplete_light.shortcuts as autocomplete_light

from .models import OnTheFly


class OnTheFlyForm(autocomplete_light.ModelForm):
    class Meta:
        model = OnTheFly
        fields = ('name', 'other_fly')
        widgets = {
            'other_fly': autocomplete_light.ChoiceWidget(
                'OnTheFlyAutocomplete', widget_attrs={
                    'data-widget-bootstrap': 'onthefly-widget'})
        }



from django.db import models


class Dummy(models.Model):
    parent = models.ForeignKey('self', null=True, blank=True)
    country = models.ForeignKey('cities_light.country')
    region = models.ForeignKey('cities_light.region')

    def __unicode__(self):
        return '%s %s' % (self.country, self.region)



from django.apps import AppConfig


class AppConfigWithoutRegistryFile(AppConfig):
    name = 'autocomplete_light.example_apps.app_config_without_registry_file'

    def ready(self):
        import autocomplete_light.shortcuts as autocomplete_light

        class AppConfigWithoutRegistryAutocomplete(
            autocomplete_light.AutocompleteListBase):
        
            choices = ['a', 'b']

        autocomplete_light.register(AppConfigWithoutRegistryAutocomplete)



import autocomplete_light
from django import forms
from models import Taggable


class TaggableForm(forms.ModelForm):
    class Meta:
        model = Taggable
        widgets = {
            'tags': autocomplete_light.TextWidget('TagAutocomplete'),
        }



from __future__ import unicode_literals

import unittest
import pytest

from django import VERSION
from django.utils.encoding import force_text

from autocomplete_light.example_apps.autocomplete_test_case_app.models import (
        NonIntegerPk, SubGroup, CustomSchema, CustomIntegerPk, Caps)
from .case import *


class AutocompleteModelMock(autocomplete_light.AutocompleteModelBase):
    limit_choices = 2
    choices = User.objects.all()
    search_fields = ('username', 'email')


class FormMock(forms.Form):
    x = forms.ModelChoiceField(queryset=AutocompleteModelMock.choices,
        widget=autocomplete_light.ChoiceWidget(
            autocomplete=AutocompleteModelMock))


class MultipleFormMock(forms.Form):
    x = forms.ModelMultipleChoiceField(queryset=AutocompleteModelMock.choices,
        widget=autocomplete_light.MultipleChoiceWidget(
            autocomplete=AutocompleteModelMock))


class AutocompleteModelTestCase(AutocompleteTestCase):
    autocomplete_mock = AutocompleteModelMock

    def assert_choices_equal(self, result, test):
        self.assertEqual([x.pk for x in result],
                         [x.pk for x in test['expected']])

    def get_choices_for_values_tests(self):
        return (
            {
                'fixture': [1, 4],
                'expected': [
                    self.abe,
                    self.john,
                ]
            },
            {
                'fixture': [4, 1],
                'expected': [
                    self.john,
                    self.abe,
                ]
            },
        )

    def get_choices_for_request_tests(self):
        return (
            {
                'fixture': make_get_request('q=j'),
                'kwargs': {
                    'limit_choices': 5,
                },
                'expected': [
                    self.jack,
                    self.james,
                    self.john,
                ]
            },
            {
                'fixture': make_get_request('q=j'),
                'kwargs': {
                    'order_by': '-username',
                    'limit_choices': 5,
                },
                'expected': [
                    self.john,
                    self.james,
                    self.jack,
                ]
            },
            {
                'fixture': make_get_request('q=j'),
                'kwargs': {
                    'order_by': ('-email', 'username'),
                    'limit_choices': 5,
                },
                'expected': [
                    self.james,
                    self.john,
                    self.jack,
                ]
            },
            {
                'fixture': make_get_request('q=sale'),
                'expected': [
                    self.abe,
                    self.james,
                ]
            },
            {
                'fixture': make_get_request(),
                'expected': [
                    self.abe,
                    self.jack,
                ]
            },
        )

    def get_validate_tests(self):
        return (
            {
                'fixture': [1, 4],
                'expected': True,
            },
            {
                'fixture': [1, 4, 123],
                'expected': False,
            },
        )

    def get_autocomplete_html_tests(self):
        return (
            {
                'fixture': make_get_request('q=j'),
                'expected': ''.join([
                    '<span data-value="%s">%s</span>' % (
                        self.jack.pk, force_text(self.jack)),
                    '<span data-value="%s">%s</span>' % (
                        self.james.pk, force_text(self.james)),
                ])
            },
            {
                'fixture': make_get_request(),
                'expected': ''.join([
                    '<span data-value="%s">%s</span>' % (
                        self.abe.pk, force_text(self.abe)),
                    '<span data-value="%s">%s</span>' % (
                        self.jack.pk, force_text(self.jack)),
                ])
            },
        )

    def get_widget_tests(self):
        return (
            {
                'form_class': FormMock,
                'fixture': 'x=4',
                'expected_valid': True,
                'expected_data': self.john,
            },
            {
                'form_class': FormMock,
                'fixture': 'x=3&x=4',
                'expected_valid': True,
                'expected_data': self.john,
            },
            {
                'fixture': 'x=abc',
                'expected_valid': False,
            },
            {
                'form_class': MultipleFormMock,
                'fixture': 'x=4&x=2',
                'expected_valid': True,
                'expected_data': [self.jack, self.john],
            },
            {
                'fixture': 'x=abc&x=2',
                'expected_valid': False,
            },
        )

    @unittest.skipIf(VERSION < (1, 8), 'Django < 1.8')
    def test_queryset_mistake(self):
        class Test(autocomplete_light.AutocompleteModelBase):
            choices = NonIntegerPk.objects.select_related('artist')

        fixture = Test(values=[NonIntegerPk.objects.create(name='bal').pk])
        results = fixture.choices_for_values()

        with pytest.raises(Exception):
            len(results)

    def test_ambiguous_column_name(self):
        class Test(autocomplete_light.AutocompleteModelBase):
            choices = NonIntegerPk.objects.select_related('noise')

        fixture = Test(values=[NonIntegerPk.objects.create(name='bal').pk])
        # Call len to force evaluate queryset
        len(fixture.choices_for_values())

    def test_inherited_model_ambiguous_column_name(self):
        subgroup = SubGroup.objects.create(name='test')

        class Test(autocomplete_light.AutocompleteModelBase):
            choices = SubGroup.objects.all()

        fixture = Test(values=[subgroup.pk])
        assert fixture.choices_for_values()[0] == subgroup

    def test_custom_table_and_name(self):
        obj = CustomSchema.objects.create(name='test')

        class Test(autocomplete_light.AutocompleteModelBase):
            choices = CustomSchema.objects.all()

        fixture = Test(values=[obj.pk])
        assert fixture.choices_for_values()[0] == obj

    def test_primary_key_zero(self):
        obj = CustomIntegerPk.objects.create(id=0)

        class Test(autocomplete_light.AutocompleteModelBase):
            choices = CustomIntegerPk.objects.all()

        fixture = Test(values=[obj.pk])
        self.assertEqual(list(fixture.choices_for_values()), [obj])

    def test_list_ordering(self):
        class Fixture(AutocompleteModelMock):
            order_by = ['id']

        fixture = Fixture(values=[1, 4])
        self.assertEqual(list(fixture.choices_for_values()), [self.abe, self.john])

    def test_caps(self):
        obj = Caps.objects.create(id=7, name='test')

        class Test(autocomplete_light.AutocompleteModelBase):
            choices = Caps.objects.all()

        fixture = Test(values=[obj.pk])
        assert fixture.choices_for_values()[0] == obj

    def test_choice_with_ordering(self):
        obj = User.objects.get(username='Elton')

        class Test(autocomplete_light.AutocompleteModelBase):
            choices = User.objects.all()

        t = Test(values=[obj.pk])
        # Current value in first position
        ch_id = t.order_choices(t.choices)[0].id
        assert ch_id == obj.id



import autocomplete_light.shortcuts as autocomplete_light
from django.test import TestCase
from lxml.cssselect import CSSSelector
from lxml.html import etree

from ..example_apps.basic.models import FkModel

try:
    from unittest import mock
except ImportError:  # python2
    import mock





class LazyAutocomplete(autocomplete_light.AutocompleteModelBase):
    pass


class WidgetBaseMixin(object):
    widget_class = autocomplete_light.WidgetBase
    fixtures = ['security_test']

    def autocomplete_input(self, et):
        return CSSSelector('input.autocomplete')(et)[0]

    def test_init_with_registry(self):
        registry = autocomplete_light.AutocompleteRegistry()
        registry.register(FkModel, name='TestAutocomplete')

        widget = self.widget_class('TestAutocomplete', registry=registry)
        self.assertEqual(widget.autocomplete.__name__, 'TestAutocomplete')

    def test_init_without_registry(self):
        widget = self.widget_class('FkModelAutocomplete')
        self.assertEqual(widget.autocomplete.model, FkModel)

    def test_widget_js_attributes_deprecation(self):
        with self.assertRaises(PendingDeprecationWarning):
            self.widget_class(widget_js_attributes={'foo': 'bar'})

    def test_autocomplete_js_attributes_deprecation(self):
        with self.assertRaises(PendingDeprecationWarning):
            self.widget_class(autocomplete_js_attributes={'foo': 'bar'})

    @mock.patch('autocomplete_light.widgets.render_to_string')
    def test_widget_template(self, render_to_string):
        widget = self.widget_class('FkModelAutocomplete',
            widget_template='foo.html')
        widget.render('somewidget', None)
        render_to_string.assert_called_with('foo.html', mock.ANY)

    @mock.patch('autocomplete_light.widgets.render_to_string')
    def test_autocomplete_widget_template(self, render_to_string):
        class Autocomplete(autocomplete_light.AutocompleteListBase):
            widget_template='bar.html'
            choices = ['a', 'b']

        widget = self.widget_class(Autocomplete)
        widget.render('somewidget', [])
        render_to_string.assert_called_with('bar.html', mock.ANY)

    @mock.patch('autocomplete_light.widgets.render_to_string')
    def test_base_context(self, render_to_string):
        widget = self.widget_class('FkModelAutocomplete')
        widget.render('somewidget', None)
        render_to_string.assert_called_with(
            'autocomplete_light/widget.html', {
                'widget': widget,
                'choices': mock.ANY,
                'autocomplete': mock.ANY,
                'attrs': mock.ANY,
                'widget_attrs': mock.ANY,
                'name': 'somewidget',
                'values': [],
            })

    @mock.patch('autocomplete_light.widgets.render_to_string')
    def test_extra_context(self, render_to_string):
        widget = self.widget_class('FkModelAutocomplete',
                                   extra_context={'foo': 'bar'})

        widget.render('somewidget', None)

        render_to_string.assert_called_with(
            'autocomplete_light/widget.html', {
                'widget': widget,
                'choices': mock.ANY,
                'autocomplete': mock.ANY,
                'attrs': mock.ANY,
                'widget_attrs': mock.ANY,
                'name': 'somewidget',
                'values': [],
                'foo': 'bar',
            })

    def test_input_placeholder_attr(self):
        widget = self.widget_class('FkModelAutocomplete',
                                   attrs={'placeholder': 'foo'})
        html = widget.render('somewidget', None)
        et = etree.XML(html)

        self.assertEqual(self.autocomplete_input(et).attrib['placeholder'],
                         'foo')

    def test_widget_attrs(self):
        widget = self.widget_class('FkModelAutocomplete',
                                   widget_attrs={'class': 'foo'})
        html = widget.render('somewidget', None)
        et = etree.XML(html)
        self.assertIn('foo', et.attrib['class'])

        # This was originally masked from the test suite because method
        # definition was repeated
        widget = self.widget_class('FkModelAutocomplete',
            widget_attrs={'data-widget-foo': 'bar', 'class':'foobar'})
        html = widget.render('somewidget', None)
        et = etree.fromstring(html)
        self.assertEquals(et.attrib['data-widget-foo'], 'bar')
        self.assertIn('foobar', et.attrib['class'])
        self.assertIn('autocomplete-light-widget', et.attrib['class'])

    def test_lazy_autocomplete_init(self):
        registry = autocomplete_light.AutocompleteRegistry()

        try:
            self.widget_class('LazyAutocomplete', registry=registry)
        except autocomplete_light.AutocompleteNotRegistered:
            self.fail('WidgetBase initialization should not trigger registry '
                      'access')

    def test_lazy_autcomplete_access(self):
        registry = autocomplete_light.AutocompleteRegistry()

        widget = self.widget_class('LazyAutocomplete', registry=registry)

        try:
            widget.autocomplete
            self.fail('Should raise AutocompleteNotRegistered on unregistered '
                      'LazyAutocomplete')
        except autocomplete_light.AutocompleteNotRegistered:
            pass

        registry.register(LazyAutocomplete)
        self.assertIn('LazyAutocomplete', registry.keys())

        try:
            widget.autocomplete
        except autocomplete_light.AutocompleteNotRegistered:
            self.fail('widget.autocomplete access should not raise '
                      'AutocompleteNotRegistered')

    def test_value_out_of_queryset(self):
        widget = self.widget_class('ItemAutocomplete')
        html = widget.render('somewidget', [1, 2])
        span = etree.fromstring(html)

        choices = CSSSelector('[data-value]')(span)

        self.assertEqual(len(choices), 1)
        self.assertEqual(int(choices[0].attrib['data-value']), 1)

class ChoiceWidgetTestCase(WidgetBaseMixin, TestCase):
    widget_class = autocomplete_light.ChoiceWidget

    def test_attrs_copy_class(self):
        widget = self.widget_class('A2')
        html = widget.render('taggit', value='oky')
        et = etree.XML(html)
        self.assertTrue(et.find('input').get('data-class-defined'), '1')

    def test_attrs_copy_init(self):
        widget = self.widget_class('A2')
        html = widget.render('taggit', value='oky')
        et = etree.XML(html)

        self.assertTrue(et.find('input').get('data-init-defined'), '2')


class MultipleChoiceWidgetTestCase(WidgetBaseMixin, TestCase):
    widget_class = autocomplete_light.MultipleChoiceWidget


class TextWidgetTestCase(WidgetBaseMixin, TestCase):
    widget_class = autocomplete_light.TextWidget

    def autocomplete_input(self, et):
        return et

    def test_extra_context(self):
        pass  # no template for TextWidget

    def test_widget_template(self):
        pass  # no template for TextWidget

    def test_base_context(self):
        pass  # no template for TextWidget

    def test_autocomplete_widget_template(self):
        pass  # no template for TextWidget

    def test_widget_attrs(self):
        pass  # no widget_attrs for TextWidget

    def test_value_out_of_queryset(self):
        pass  # no queryset for text widget

    def test_widget_attrs_copy(self):
        # Test case for GH269
        widget = self.widget_class('B')
        html = widget.render('taggit', value='Cued Speech, languages')
        et = etree.XML(html)
        self.assertTrue('value' in et.attrib)

        html = widget.render('taggit', None)
        et = etree.XML(html)
        self.assertFalse('value' in et.attrib)

    def test_attrs_copy_class(self):
        widget = self.widget_class('B2')
        html = widget.render('taggit', value='oky')
        et = etree.XML(html)
        self.assertTrue('data-class-defined' in et.attrib)

    def test_attrs_copy_init(self):
        widget = self.widget_class('B2')
        html = widget.render('taggit', value='oky')
        et = etree.XML(html)
        self.assertTrue('data-init-defined' in et.attrib)



import autocomplete_light.shortcuts as autocomplete_light
from django import VERSION

from .models import *

try:
    import genericm2m
except ImportError:
    genericm2m = None

try:
    import taggit
except ImportError:
    taggit = None




class DjangoCompatMeta:
    if VERSION >= (1, 6):
        fields = '__all__'


class FkModelForm(autocomplete_light.ModelForm):
    class Meta(DjangoCompatMeta):
        model = FkModel


class OtoModelForm(autocomplete_light.ModelForm):
    class Meta(DjangoCompatMeta):
        model = OtoModel


class MtmModelForm(autocomplete_light.ModelForm):
    class Meta(DjangoCompatMeta):
        model = MtmModel


class GfkModelForm(autocomplete_light.ModelForm):
    class Meta(DjangoCompatMeta):
        model = GfkModel


if genericm2m:
    class GmtmModelForm(autocomplete_light.ModelForm):
        class Meta(DjangoCompatMeta):
            model = GmtmModel


if taggit:
    class TaggitModelForm(autocomplete_light.ModelForm):
        class Meta(DjangoCompatMeta):
            model = TaggitModel



from autocomplete_light.exceptions import AutocompleteNotRegistered
from django import http
from django.utils.encoding import force_text
from django.views import generic
from django.views.generic import base

__all__ = ['AutocompleteView', 'RegistryView', 'CreateView']


class GetRegistryMixin(object):
    def get_registry(self):
        if getattr(self, '_registry', None) is None:
            from autocomplete_light.registry import registry
            self._registry = registry
        return self._registry


class RegistryView(GetRegistryMixin, base.TemplateView):
    template_name = 'autocomplete_light/registry.html'

    def get(self, request, *args, **kwargs):
        if not request.user.is_superuser:
            return http.HttpResponseForbidden()
        return super(RegistryView, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        return {
            'registry': self.get_registry(),
            'registry_items': self.get_registry().items(),
        }


class AutocompleteView(GetRegistryMixin, generic.View):
    """Simple view that routes the request to the appropriate autocomplete."""

    def get(self, request, *args, **kwargs):
        """
        Return an HttpResponse with the return value of
        autocomplete.autocomplete_html().

        This view is called by the autocomplete script, it is expected to
        return the rendered autocomplete box contents.

        To do so, it gets the autocomplete class from the registry, given the
        url keyword argument autocomplete, that should be the autocomplete
        name.

        Then, it instanciates the autocomplete with no argument as usual, and
        calls autocomplete.init_for_request, passing all arguments it recieved.

        Finnaly, it makes an HttpResponse with the result of
        autocomplete.autocomplete_html(). The javascript will use that to
        fill the autocomplete suggestion box.
        """
        try:
            autocomplete_class = self.get_registry()[kwargs['autocomplete']]
        except AutocompleteNotRegistered:
            return http.HttpResponseNotFound()
        autocomplete = autocomplete_class(request=request)
        return http.HttpResponse(autocomplete.autocomplete_html())

    def post(self, request, *args, **kwargs):
        """
        Just proxy autocomplete.post().

        This is the key to communication between the autocomplete and the
        widget in javascript. You can use it to create results and such.
        """
        autocomplete_class = self.get_registry()[kwargs['autocomplete']]
        autocomplete = autocomplete_class()
        return autocomplete.post(request, *args, **kwargs)


class CreateView(generic.CreateView):
    """Simple wrapper for generic.CreateView, that responds to _popup."""

    def is_popup(self):
        return self.request.GET.get('_popup', False)

    def respond_script(self, obj=None):
        if obj is None:
            obj = self.object

        html = []
        html.append('<script type="text/javascript">')
        html.append('opener.dismissAddAnotherPopup( window, "%s", "%s" );' % (
            force_text(obj.pk), force_text(obj).replace('"', '\\"')))
        html.append('</script>')

        html = ''.join(html)

        return http.HttpResponse(html, status=201)

    def form_valid(self, form):
        """ If request.GET._popup, return some javascript. """
        if self.is_popup():
            self.success_url = '/'  # avoid ImproperlyConfigured

        response = super(CreateView, self).form_valid(form)

        if not self.is_popup():
            return response

        return self.respond_script()



import autocomplete_light.shortcuts as al

from autocomplete_light.compat import url, urls
from django.views import generic

from .forms import NonAdminAddAnotherModelForm
from .models import NonAdminAddAnotherModel

urlpatterns = urls([
    url(r'$', al.CreateView.as_view(
        model=NonAdminAddAnotherModel, form_class=NonAdminAddAnotherModelForm),
        name='non_admin_add_another_model_create'),
    url(r'(?P<pk>\d+)/$', generic.UpdateView.as_view(
        model=NonAdminAddAnotherModel, form_class=NonAdminAddAnotherModelForm),
        name='non_admin_add_another_model_update'),
])



"""
Provide tools to enable nice autocompletes in your Django project.
"""
import django

if django.VERSION < (1, 9):
    from .shortcuts import *  # noqa

default_app_config = 'autocomplete_light.apps.AutocompleteLightConfig'

__version__ = (2, 3, 4)



import unittest

import autocomplete_light.shortcuts as autocomplete_light
import django
from django.contrib.auth.models import User
from django.db import models
from django.test import TestCase


class Noname(models.Model):
    number = models.CharField(max_length=100)


class Foo(models.Model):
    name = models.CharField(max_length=100)


class Bar(autocomplete_light.AutocompleteModelBase):
    pass


class Generic(autocomplete_light.AutocompleteGenericBase):
    choices = (
        User.objects.all(),
    )
    search_fields = (
        ('username',),
    )


class RegistryTestCase(TestCase):
    def setUp(self):
        self.registry = autocomplete_light.AutocompleteRegistry()

    def test_register_model(self):
        self.registry.register(Foo)
        self.assertIn('FooAutocomplete', self.registry.keys())

    def test_register_model_and_autocomplete(self):
        self.registry.register(Foo, Bar)
        self.assertIn('FooBar', self.registry.keys())

    def test_register_autocomplete(self):
        self.registry.register(Bar)
        self.assertIn('Bar', self.registry.keys())

    def test_unregister(self):
        self.registry.register(Bar)
        self.registry.unregister('Bar')
        self.assertEqual(list(self.registry.keys()), [])

    def test_register_with_kwargs(self):
        choices = ['foo']
        self.registry.register(Foo, search_name='search_name', choices=choices)
        self.assertEqual(self.registry['FooAutocomplete'].search_name,
            'search_name')
        self.assertEqual(self.registry['FooAutocomplete'].choices, choices)

    def test_register_with_custom_autocomplete_model_base(self):
        class NewBase(autocomplete_light.AutocompleteModelBase):
            new_base = True

        self.registry.autocomplete_model_base = NewBase
        self.registry.register(Foo)
        self.assertEqual(NewBase, self.registry['FooAutocomplete'].__base__)
        self.assertTrue(self.registry['FooAutocomplete'].new_base)

    def test_register_with_autocomplete_and_kwargs(self):
        self.registry.register(Foo, Bar, search_name='search_name')
        self.assertEqual(self.registry['FooBar'].search_name,
            'search_name')

    def test_register_with_custom_name(self):
        self.registry.register(Foo, Bar, name='BarFoo')
        self.assertIn('BarFoo', self.registry.keys())
        self.assertEqual(self.registry['BarFoo'].__name__, 'BarFoo')

    def test_register_no_name_fail(self):
        try:
            self.registry.register(Noname)
            self.fail('Should raise an exception when registering noname')
        except:
            pass

    def test_register_no_name_pass(self):
        self.registry.register(Noname, search_fields=('number',))

    def test_register_generic_with_custom_name(self):
        self.registry.register(Generic, name='foo')
        self.assertTrue('foo' in self.registry.keys())

    def test_raise_AutocompleteNotRegistered(self):
        try:
            self.registry['NotRegistered']
            self.fail('Should raise AutocompleteNotRegistered')
        except autocomplete_light.AutocompleteNotRegistered:
            pass

    def test_raise_NoGenericAutocompleteRegistered(self):
        self.assertRaises(autocomplete_light.NoGenericAutocompleteRegistered,
                          self.registry.autocomplete_for_generic)

    def test_autocomplete_for_model(self):
        class FirstAutocomplete(autocomplete_light.AutocompleteModelBase):
            pass

        class SecondAutocomplete(autocomplete_light.AutocompleteModelBase):
            pass

        self.registry.register(Foo, FirstAutocomplete)
        self.registry.register(Foo, SecondAutocomplete)

        self.assertTrue(issubclass(
            self.registry.autocomplete_for_model(Foo), FirstAutocomplete))

    def test_autocomplete_for_generic(self):
        class FirstAutocomplete(Generic):
            pass

        class SecondAutocomplete(Generic):
            pass

        self.registry.register(FirstAutocomplete)
        self.registry.register(SecondAutocomplete)

        self.assertTrue(issubclass(
            self.registry.autocomplete_for_generic(), FirstAutocomplete))

    def test_register_model_as_string_app_name_dot_model_name(self):
        self.registry.register('auth.User')
        self.assertIn('UserAutocomplete', self.registry.keys())

    def test_register_model_as_string_invalid_app_name_dot_model_name(self):
        try:
            self.registry.register('invalid_appname.User')
            self.fail('Should raise an exception because "LookupError: No installed app with label invalid_appname"')
        except LookupError:
            pass

    def test_register_model_as_string_app_name_dot_invalid_model_name(self):
        try:
            self.registry.register('django.contrib.auth.models.UserWrong')
            self.fail(
                'Should raise an exception because "ImportError: Module '
                '"django.contrib.auth.models" does not define a \"UserWrong\" attribute/class"')
        except ImportError:
            pass

    def test_register_model_as_string_full_dot_path_to_model(self):
        self.registry.register('django.contrib.auth.models.User')
        self.assertIn('UserAutocomplete', self.registry.keys())

    def test_register_model_as_string_full_dot_path_to_non_model(self):
        try:
            self.registry.register('django.contrib.auth.forms.UserCreationForm')
            self.fail('Should raise an exception NonDjangoModelSubclassException because '
                      'UserCreationForm not is subclass of django Model')
        except autocomplete_light.NonDjangoModelSubclassException:
            pass

    def test_register_model_as_string_invalid_full_dot_path_to_non_model(self):
        try:
            self.registry.register('django.invalid_path.UserCreationForm')
            self.fail('Should raise an exception because is invalid dot path to Model')
        except ImportError:
            pass


class RegistryGetAutocompleteFromArgTestCase(TestCase):
    def setUp(self):
        self.registry = autocomplete_light.AutocompleteRegistry()
        self.registry.register(Foo)
        self.registry.register(Generic)

    def test_from_string(self):
        a = self.registry.get_autocomplete_from_arg('FooAutocomplete')
        self.assertEqual(a.model, Foo)

    def test_from_model(self):
        a = self.registry.get_autocomplete_from_arg(Foo)
        self.assertEqual(a.model, Foo)

    def test_from_model_instance(self):
        a = self.registry.get_autocomplete_from_arg(Foo())
        self.assertEqual(a.model, Foo)

    def test_from_autocomplete_instance(self):
        a = self.registry.get_autocomplete_from_arg(Generic)
        self.assertEqual(a, Generic)

    def test_default_generic(self):
        a = self.registry.get_autocomplete_from_arg()
        self.assertTrue(issubclass(a, Generic))

    def test_model_picked_up_from_autocomplete_class_model(self):
        # GitHub issue #313
        class TestModel(models.Model):
            name = models.CharField(max_length=100)

        class XAutocomplete(autocomplete_light.AutocompleteModelBase):
            model = TestModel

        self.registry.register(XAutocomplete)
        result = self.registry.get_autocomplete_from_arg(TestModel)

        assert result
        assert issubclass(result, XAutocomplete)

    def test_model_picked_up_from_autocomplete_class_choices_model(self):
        class TestModel(models.Model):
            name = models.CharField(max_length=100)

        class YAutocomplete(autocomplete_light.AutocompleteModelBase):
            choices = TestModel.objects.all()

        self.registry.register(YAutocomplete)
        result = self.registry.get_autocomplete_from_arg(TestModel)

        assert result
        assert issubclass(result, YAutocomplete)

    def test_registering_autocomplete_without_model_name_as_prefix(self):
        class TestModel(models.Model):
            name = models.CharField(max_length=100)

        class Base(autocomplete_light.AutocompleteModelBase):
            pass

        class BarAutocomplete(Base):
            model = TestModel
            choices = TestModel.objects.all()

        self.registry.register(BarAutocomplete)
        assert 'BarAutocomplete' in self.registry
        result = self.registry.get_autocomplete_from_arg(TestModel)
        assert result
        assert issubclass(result, BarAutocomplete)


@unittest.skipIf(django.VERSION < (1, 7), 'require django 1.7')
class AppConfigSupportTestCase(TestCase):
    def test_appconfig_with_registry_file(self):
        self.assertIsInstance(autocomplete_light.registry['AppConfigWithRegistryAutocomplete'](),
                             autocomplete_light.AutocompleteListBase)

    def test_appconfig_without_registry_file(self):
        self.assertIsInstance(autocomplete_light.registry['AppConfigWithoutRegistryAutocomplete'](),
                              autocomplete_light.AutocompleteListBase)



from .autocomplete.shortcuts import *
from .contrib.taggit_field import TaggitField, TaggitWidget
from .exceptions import (AutocompleteArgNotUnderstood,
                         AutocompleteChoicesMustBeQuerySet,
                         AutocompleteLightException, AutocompleteNotRegistered,
                         NoGenericAutocompleteRegistered, NonDjangoModelSubclassException)
from .fields import *
from .forms import *
from .registry import AutocompleteRegistry, autodiscover, register, registry
from .settings import *
from .views import AutocompleteView, CreateView, RegistryView
from .widgets import ChoiceWidget, MultipleChoiceWidget, TextWidget, WidgetBase

try:
    import taggit
except ImportError:
    pass
else:
    from .contrib.taggit_field import TaggitField, TaggitWidget
